'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var timekeeper = _interopDefault(require('timekeeper'));
var chai = require('chai');
var lodash = require('lodash');
var configureStore = _interopDefault(require('redux-mock-store'));
var sinon = require('sinon');

var CALL_API = Symbol('CALL_API');
var REDUCER_PATH = 'api_calls';
var ACTION_FETCH_START = '@@api/FETCH_START';
var ACTION_FETCH_COMPLETE = '@@api/FETCH_COMPLETE';
var ACTION_FETCH_FAILURE = '@@api/FETCH_FAILURE';
var __original_CALL_API;
var CALL_API_stubbed;


var __original_REDUCER_PATH;
var REDUCER_PATH_stubbed;


var __original_ACTION_FETCH_START;
var ACTION_FETCH_START_stubbed;


var __original_ACTION_FETCH_COMPLETE;
var ACTION_FETCH_COMPLETE_stubbed;


var __original_ACTION_FETCH_FAILURE;
var ACTION_FETCH_FAILURE_stubbed;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) {
  return typeof obj;
} : function (obj) {
  return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj;
};













var defineProperty = function (obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
};

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};

var get$1 = function get$1(object, property, receiver) {
  if (object === null) object = Function.prototype;
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent === null) {
      return undefined;
    } else {
      return get$1(parent, property, receiver);
    }
  } else if ("value" in desc) {
    return desc.value;
  } else {
    var getter = desc.get;

    if (getter === undefined) {
      return undefined;
    }

    return getter.call(receiver);
  }
};

















var set = function set(object, property, value, receiver) {
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent !== null) {
      set(parent, property, value, receiver);
    }
  } else if ("value" in desc && desc.writable) {
    desc.value = value;
  } else {
    var setter = desc.set;

    if (setter !== undefined) {
      setter.call(receiver, value);
    }
  }

  return value;
};

var makeStartErrorAction = function makeStartErrorAction(api) {
  return function (error) {
    return {
      type: ACTION_FETCH_START,
      error: true,
      payload: _extends({
        error: error
      }, api)
    };
  };
};

var makeStartAction = function makeStartAction(api) {
  return function () {
    return {
      type: ACTION_FETCH_START,
      payload: _extends({}, api, {
        requestedAt: Date.now()
      })
    };
  };
};

var makeSuccessAction = function makeSuccessAction(api) {
  return function (json) {
    return {
      type: ACTION_FETCH_COMPLETE,
      payload: _extends({}, api, {
        json: json,
        respondedAt: Date.now()
      })
    };
  };
};

var makeFailureAction = function makeFailureAction(api) {
  return function (json) {
    return {
      type: ACTION_FETCH_FAILURE,
      payload: _extends({}, api, {
        json: json,
        respondedAt: Date.now()
      })
    };
  };
};

var __original_makeStartErrorAction;
var makeStartErrorAction_stubbed;
function stub_makeStartErrorAction(__stubValue__) { if (!makeStartErrorAction_stubbed) { __original_makeStartErrorAction = makeStartErrorAction; makeStartErrorAction_stubbed = true; } makeStartErrorAction = __stubValue__; }
function reset_makeStartErrorAction() { if (makeStartErrorAction_stubbed) { makeStartErrorAction_stubbed = false; makeStartErrorAction = __original_makeStartErrorAction; } }
var __original_makeStartAction;
var makeStartAction_stubbed;
function stub_makeStartAction(__stubValue__) { if (!makeStartAction_stubbed) { __original_makeStartAction = makeStartAction; makeStartAction_stubbed = true; } makeStartAction = __stubValue__; }
function reset_makeStartAction() { if (makeStartAction_stubbed) { makeStartAction_stubbed = false; makeStartAction = __original_makeStartAction; } }
var __original_makeSuccessAction;
var makeSuccessAction_stubbed;
function stub_makeSuccessAction(__stubValue__) { if (!makeSuccessAction_stubbed) { __original_makeSuccessAction = makeSuccessAction; makeSuccessAction_stubbed = true; } makeSuccessAction = __stubValue__; }
function reset_makeSuccessAction() { if (makeSuccessAction_stubbed) { makeSuccessAction_stubbed = false; makeSuccessAction = __original_makeSuccessAction; } }
var __original_makeFailureAction;
var makeFailureAction_stubbed;
function stub_makeFailureAction(__stubValue__) { if (!makeFailureAction_stubbed) { __original_makeFailureAction = makeFailureAction; makeFailureAction_stubbed = true; } makeFailureAction = __stubValue__; }
function reset_makeFailureAction() { if (makeFailureAction_stubbed) { makeFailureAction_stubbed = false; makeFailureAction = __original_makeFailureAction; } }

describe('Action Creators', function () {
  before(function () {
    timekeeper.freeze(Date.now());
  });

  after(function () {
    timekeeper.reset();
  });

  describe('makeStartErrorAction', function () {
    var action = void 0;
    var SAMPLE_API = {
      name: 'SAMPLE',
      endpoint: 'http://example.com'
    };
    var SAMPLE_ERROR = new Error('some error');

    before(function () {
      action = makeStartErrorAction(SAMPLE_API)(SAMPLE_ERROR);
    });

    it('should have type of FETCH_START', function () {
      chai.expect(action).to.have.property('type', ACTION_FETCH_START);
    });

    it('should have property error=true', function () {
      chai.expect(action).to.have.property('error', true);
    });

    it('should include api config in payload', function () {
      lodash.forEach(SAMPLE_API, function (value, key) {
        chai.expect(action.payload).to.have.property(key, value);
      });
    });

    it('should include error defailt in payload under `error` property', function () {
      chai.expect(action.payload.error).to.equal(SAMPLE_ERROR);
    });
  });

  describe('makeStartAction', function () {
    var action = void 0;
    var SAMPLE_API = {
      name: 'SAMPLE',
      endpoint: 'http://example.com'
    };

    before(function () {
      action = makeStartAction(SAMPLE_API)();
    });

    it('should have type of FETCH_START', function () {
      chai.expect(action).to.have.property('type', ACTION_FETCH_START);
    });

    it('should not have property error', function () {
      chai.expect(action).to.not.have.property('error');
    });

    it('should include api config in payload', function () {
      lodash.forEach(SAMPLE_API, function (value, key) {
        chai.expect(action.payload).to.have.property(key, value);
      });
    });

    it('should include requested time as `requestedAt` prop', function () {
      chai.expect(action.payload.requestedAt).to.equal(Date.now());
    });
  });

  describe('makeSuccessAction', function () {
    var action = void 0;
    var SAMPLE_API = {
      name: 'SAMPLE',
      endpoint: 'http://example.com'
    };
    var SAMPLE_RESPONSE = {
      key: 'value'
    };

    before(function () {
      action = makeSuccessAction(SAMPLE_API)(SAMPLE_RESPONSE);
    });

    it('should have type of FETCH_COMPLETE', function () {
      chai.expect(action).to.have.property('type', ACTION_FETCH_COMPLETE);
    });

    it('should not have property error', function () {
      chai.expect(action).to.not.have.property('error');
    });

    it('should include api config in payload', function () {
      lodash.forEach(SAMPLE_API, function (value, key) {
        chai.expect(action.payload).to.have.property(key, value);
      });
    });

    it('should include json response in payload under `json` property', function () {
      chai.expect(action.payload.json).to.equal(SAMPLE_RESPONSE);
    });
  });

  describe('makeFailureAction', function () {
    var action = void 0;
    var SAMPLE_API = {
      name: 'SAMPLE',
      endpoint: 'http://example.com'
    };
    var SAMPLE_ERROR = {
      error: 'reason'
    };

    before(function () {
      action = makeFailureAction(SAMPLE_API)(SAMPLE_ERROR);
    });

    it('should have type of FETCH_FAIL', function () {
      chai.expect(action).to.have.property('type', ACTION_FETCH_FAILURE);
    });

    it('should not have property error', function () {
      chai.expect(action).to.not.have.property('error');
    });

    it('should include api config in payload', function () {
      lodash.forEach(SAMPLE_API, function (value, key) {
        chai.expect(action.payload).to.have.property(key, value);
      });
    });

    it('should include json response in payload under `json` property', function () {
      chai.expect(action.payload.json).to.equal(SAMPLE_ERROR);
    });
  });
});

var defaultAdapter = (function (_ref) {
  var endpoint = _ref.endpoint;
  var others = _ref.others;

  return fetch(endpoint, others);
});

describe('defaultAdapter', function () {
  it('should be a function', function () {
    chai.expect(defaultAdapter).to.be.an.instanceOf(Function);
  });

  it('should return a promise', function () {
    chai.expect(defaultAdapter({})).to.be.an.instanceOf(Promise);
  });
});

var get$2 = (function (array, defaulValue) {
  return function (state) {
    var finalValue = array.reduce(function (value, nextProp) {
      if (typeof value === 'undefined' || value === null) {
        return;
      }
      return value[nextProp];
    }, state);

    if (typeof finalValue === 'undefined') {
      return defaulValue;
    }

    return finalValue;
  };
});

var assertTypes = (function (value, types, msg) {
  if (!types.test(typeof value === "undefined" ? "undefined" : _typeof(value))) throw new Error(msg);
});

var makeSelectors = function makeSelectors(descriptors, dataSelector) {
  var keys = Object.keys(descriptors);
  var selectors = {};
  keys.forEach(function (key) {
    var fn = descriptors[key];
    assertTypes(fn, /function/, 'selector for "' + key + '" must be a function');
    selectors[key + 'Selector'] = function (state) {
      return fn(dataSelector(state));
    };
  });
  return selectors;
};

var makeFetchAction = (function (apiName, apiConfigFn) {
  var selectorDescriptor = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

  var actionCreator = function actionCreator() {
    return defineProperty({}, CALL_API, apiConfigFn.apply(undefined, arguments));
  };

  var isFetchingSelector = get$2([REDUCER_PATH, apiName, 'isFetching'], false);
  var isInvalidatedSelector = get$2([REDUCER_PATH, apiName, 'isInvalidated'], false);
  var dataSelector = get$2([REDUCER_PATH, apiName, 'data'], null);
  var errorSelector = get$2([REDUCER_PATH, apiName, 'error'], null);

  var otherSelectors = makeSelectors(selectorDescriptor, dataSelector);

  return _extends({
    actionCreator: actionCreator
  }, otherSelectors, {
    isFetchingSelector: isFetchingSelector,
    isInvalidatedSelector: isInvalidatedSelector,
    dataSelector: dataSelector,
    errorSelector: errorSelector
  });
});

describe('makeFetchAction', function () {
  it('should throw error if a selector is not a function', function () {
    chai.expect(function () {
      makeFetchAction('MUST_FAIL', function () {
        return { endpoint: 'some path' };
      }, {
        fail: 'must failed!!!'
      });
    }).to.throw('selector for "fail" must be a function');
  });

  describe('no custom selectors', function () {
    var actual = void 0;
    before(function () {
      actual = makeFetchAction('SAMPLE', lodash.constant({ endpoint: 'http://example.com' }));
    });

    it('should return actionCreator function', function () {
      chai.expect(actual).to.have.property('actionCreator').to.be.an.instanceOf(Function);
    });

    it('should return isFetchingSelector function', function () {
      chai.expect(actual).to.have.property('isFetchingSelector').to.be.an.instanceOf(Function);
    });

    it('should return dataSelector function', function () {
      chai.expect(actual).to.have.property('dataSelector').to.be.an.instanceOf(Function);
    });

    it('should return errorSelector function', function () {
      chai.expect(actual).to.have.property('errorSelector').to.be.an.instanceOf(Function);
    });

    it('should return isInvalidatedSelector function', function () {
      chai.expect(actual).to.have.property('isInvalidatedSelector').to.be.an.instanceOf(Function);
    });

    describe('selectors', function () {
      describe('isFetchingSelector', function () {
        it('should return isFetching in state if present', function () {
          chai.expect(actual.isFetchingSelector({
            api_calls: {
              SAMPLE: {
                isFetching: true
              }
            }
          })).to.equal(true);

          chai.expect(actual.isFetchingSelector({
            api_calls: {
              SAMPLE: {
                isFetching: false
              }
            }
          })).to.equal(false);
        });

        it('should return false if api was not called', function () {
          chai.expect(actual.isFetchingSelector({})).to.equal(false);
        });
      });

      describe('isInvalidatedSelector', function () {
        it('should return isInvalidated in state if present', function () {
          chai.expect(actual.isInvalidatedSelector({
            api_calls: {
              SAMPLE: {
                isInvalidated: true
              }
            }
          })).to.equal(true);

          chai.expect(actual.isInvalidatedSelector({
            api_calls: {
              SAMPLE: {
                isInvalidated: false
              }
            }
          })).to.equal(false);
        });

        it('should return false if api was not called', function () {
          chai.expect(actual.isInvalidatedSelector({})).to.equal(false);
        });
      });

      describe('dataSelector', function () {
        it('should return data in state if present', function () {
          var data = { key: 'value' };
          chai.expect(actual.dataSelector({
            api_calls: {
              SAMPLE: {
                data: data
              }
            }
          })).to.equal(data);
        });

        it('should return null if api was not called', function () {
          chai.expect(actual.dataSelector({})).to.equal(null);
        });
      });

      describe('errorSelector', function () {
        it('should return error in state if present', function () {
          var error = { error: 'value' };
          chai.expect(actual.errorSelector({
            api_calls: {
              SAMPLE: {
                error: error
              }
            }
          })).to.equal(error);
        });

        it('should return null if api was not called', function () {
          chai.expect(actual.errorSelector({})).to.equal(null);
        });
      });
    });
  });

  describe('custom selectors', function () {
    var actual = void 0;
    before(function () {
      actual = makeFetchAction('SAMPLE', lodash.constant({ endpoint: 'http://example.com' }), {
        data1: function data1(data) {
          return data.key1;
        },
        data2: function data2(data) {
          return data.key2;
        }
      });
    });

    it('should return data1Selector and data2Selector functions', function () {
      chai.expect(actual).to.have.property('data1Selector').to.be.an.instanceOf(Function);
      chai.expect(actual).to.have.property('data2Selector').to.be.an.instanceOf(Function);
    });

    describe('custom selector functions', function () {
      it('should return value based on dataSelector', function () {
        chai.expect(actual.data1Selector({
          api_calls: {
            SAMPLE: {
              data: {
                key1: 'key1',
                key2: 'key2'
              }
            }
          }
        })).to.equal('key1');

        chai.expect(actual.data2Selector({
          api_calls: {
            SAMPLE: {
              data: {
                key1: 'key1',
                key2: 'key2'
              }
            }
          }
        })).to.equal('key2');
      });
    });
  });
});

var validateApi = function validateApi(api) {
  assertTypes(api.name, /string/, 'name must be a string');
  assertTypes(api.endpoint, /string|function/, 'endpoint must be either a string or a function');
};
var __original_validateApi;
var validateApi_stubbed;
function stub_validateApi(__stubValue__) { if (!validateApi_stubbed) { __original_validateApi = validateApi; validateApi_stubbed = true; } validateApi = __stubValue__; }
function reset_validateApi() { if (validateApi_stubbed) { validateApi_stubbed = false; validateApi = __original_validateApi; } }

var middleware = (function (adapter) {
  return function (_ref) {
    var dispatch = _ref.dispatch;
    var getState = _ref.getState;
    return function (next) {
      return function (action) {
        if (!action || !action[CALL_API]) {
          next(action);
          return;
        }

        var api = action[CALL_API];
        try {
          validateApi(api);

          // valid api
          // TODO: wrap all functions with getState()
          var finalApi = api;
          dispatch(makeStartAction(finalApi)());

          // sending request
          return new Promise(function (resolve, reject) {
            var responsePromise = adapter(api);

            responsePromise.then(function (resp) {
              if (resp.ok) {
                return resp.json().then(makeSuccessAction(api));
              }

              return resp.json().then(makeFailureAction(api));
            }).then(dispatch).then(resolve);
          });
        } catch (error) {
          dispatch(makeStartErrorAction(api)(error));
          return;
        }
      };
    };
  };
});

var fakeResponse = function fakeResponse(status, _json) {
  return Promise.resolve({
    ok: status < 400,
    status: status,
    json: function json() {
      return Promise.resolve(_json);
    }
  });
};

describe('middleware', function () {
  it('should be a function', function () {
    chai.expect(middleware).to.be.a.function;
  });

  describe('with store', function () {
    var mockAdapter = sinon.stub();
    var mockStore = configureStore([middleware(mockAdapter)]);
    var mock_makeStartAction = sinon.stub();
    var mock_makeStartErrorAction = sinon.stub();
    var mock_makeSuccessAction = sinon.stub();
    var mock_makeFailureAction = sinon.stub();
    var mock_validateApi = sinon.stub();

    before(function () {
      stub_makeStartAction(mock_makeStartAction);
      stub_makeStartErrorAction(mock_makeStartErrorAction);
      stub_makeSuccessAction(mock_makeSuccessAction);
      stub_makeFailureAction(mock_makeFailureAction);
      stub_validateApi(mock_validateApi);
    });

    after(function () {
      reset_makeStartAction();
      reset_makeStartErrorAction();
      reset_makeSuccessAction();
      reset_makeFailureAction();
      reset_validateApi();
    });

    context('# when CALL_API is not passed', function () {
      var store = mockStore({});
      var actions = [{
        type: 'ANYTHING',
        payload: {}
      }, {
        type: 'NO_PAYLOAD'
      }];

      it('should dispatch original actions', function () {
        actions.forEach(store.dispatch);
        chai.expect(store.getActions()).eql(actions);
      });
    });

    context('# when API config is invalid', function () {
      var actions = void 0;

      before(function () {
        var store = mockStore({});
        mock_validateApi.throws(new Error('failed for some reason'));
        mock_makeStartErrorAction.returns(function (error) {
          return { type: 'START_ERROR', payload: error };
        });

        store.dispatch(defineProperty({}, CALL_API, {}));
        actions = store.getActions();
      });

      it('should dispatch one action created by makeStartErrorAction', function () {
        chai.expect(actions).to.have.length(1);
        var payload = actions[0].payload;

        chai.expect(payload).to.be.an.instanceOf(Error);
        chai.expect(payload).to.have.property('message', 'failed for some reason');
      });
    });

    context('# when API config is valid and API response is ok', function () {
      var actions = void 0;

      before(function () {
        timekeeper.freeze(Date.now());
      });

      after(function () {
        timekeeper.reset();
      });

      before(function (done) {
        var store = mockStore({});
        var action = defineProperty({}, CALL_API, {
          name: 'USERS',
          endpoint: 'http://localhost:9000/api/users'
        });

        // validation ok
        mock_validateApi.returns(true);
        mock_makeStartAction.returns(function () {
          return { type: 'START' };
        });
        mock_makeSuccessAction.returns(function (json) {
          return {
            type: 'SUCCESS',
            payload: json
          };
        });
        mockAdapter.returns(fakeResponse(200, { key: 'value' }));

        store.dispatch(action).then(function () {
          actions = store.getActions();
          done();
        });
      });

      it('should dispatch two actions', function () {
        chai.expect(actions).to.have.length(2);
      });

      it('should dispatch START action first', function () {
        chai.expect(actions[0]).to.have.property('type', 'START');
      });

      it('should dispatch SUCCESS action afterward', function () {
        chai.expect(actions[1]).to.have.property('type', 'SUCCESS');
      });

      it('should dispatch SUCCESS action with json response in payload', function () {
        chai.expect(actions[1].payload).to.have.property('key', 'value');
      });
    });

    context('# when API config is valid but API response is not ok', function () {
      var actions = void 0;

      before(function () {
        timekeeper.freeze(Date.now());
      });

      after(function () {
        timekeeper.reset();
      });

      before(function (done) {
        var store = mockStore({});
        var action = defineProperty({}, CALL_API, {
          name: 'USERS',
          endpoint: 'http://localhost:9000/api/users'
        });

        // validation not ok
        mock_validateApi.returns(true);
        mock_makeStartAction.returns(function () {
          return { type: 'START' };
        });
        mock_makeFailureAction.returns(function (json) {
          return {
            type: 'FAILURE',
            payload: json
          };
        });
        mockAdapter.returns(fakeResponse(400, { error: 'value' }));

        store.dispatch(action).then(function () {
          actions = store.getActions();
          done();
        });
      });

      it('should dispatch two actions', function () {
        chai.expect(actions).to.have.length(2);
      });

      it('should dispatch START action first', function () {
        chai.expect(actions[0]).to.have.property('type', 'START');
      });

      it('should dispatch FAILURE action afterward', function () {
        chai.expect(actions[1]).to.have.property('type', 'FAILURE');
      });

      it('should dispatch FAILURE action with json response in payload', function () {
        chai.expect(actions[1].payload).to.have.property('error', 'value');
      });
    });
  });
});

var handleActions = (function (handlers) {
  return function () {
    var state = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
    var action = arguments[1];

    var handler = handlers[action.type];
    if (typeof handler !== 'function') {
      return state;
    }

    return handler(state, action);
  };
});

var _handleActions;

var getName = function getName(action) {
  return action.payload.name;
};
var getRequestedAt = function getRequestedAt(action) {
  return action.payload.requestedAt;
};
var getRespondedAt = function getRespondedAt(action) {
  return action.payload.respondedAt;
};
var getJSONResponse = function getJSONResponse(action) {
  return action.payload.json;
};
var getError = function getError(action) {
  return action.payload.error;
};

var reducer = handleActions((_handleActions = {}, defineProperty(_handleActions, ACTION_FETCH_START, function (state, action) {
  var apiName = getName(action);
  var api = state[apiName];
  var next = _extends({}, api, {
    lastRequest: getRequestedAt(action),
    isFetching: !action.error,
    isInvalidated: true,
    error: action.error ? getError(action) : null
  });

  return _extends({}, state, defineProperty({}, apiName, next));
}), defineProperty(_handleActions, ACTION_FETCH_COMPLETE, function (state, action) {
  var apiName = getName(action);
  var api = state[apiName];
  var next = _extends({}, api, {
    isFetching: false,
    isInvalidated: false,
    lastResponse: getRespondedAt(action),
    data: getJSONResponse(action),
    error: null
  });

  return _extends({}, state, defineProperty({}, apiName, next));
}), defineProperty(_handleActions, ACTION_FETCH_FAILURE, function (state, action) {
  var apiName = getName(action);
  var api = state[apiName];
  var next = _extends({}, api, {
    isFetching: false,
    isInvalidated: true,
    error: getJSONResponse(action)
  });

  return _extends({}, state, defineProperty({}, apiName, next));
}), _handleActions));

describe('reducer', function () {
  before(function () {
    timekeeper.freeze(Date.now());
  });

  after(function () {
    timekeeper.reset();
  });

  describe('FETCH_START handler without `error` property', function () {
    var next = void 0;
    var state = {
      SAMPLE: {
        data: {
          key: 'old_value'
        },
        error: {
          message: 'some error has occurred'
        }
      }
    };

    before(function () {
      var api = {
        name: 'SAMPLE',
        endpoint: 'http://example.com'
      };
      next = reducer(state, makeStartAction(api)());
    });

    it('should set isFetching to true', function () {
      chai.expect(lodash.get(next, 'SAMPLE.isFetching')).to.equal(true);
    });

    it('should set isInvalidated to true', function () {
      chai.expect(lodash.get(next, 'SAMPLE.isInvalidated')).to.equal(true);
    });

    it('should set lastRequest to the requestedAt time', function () {
      chai.expect(lodash.get(next, 'SAMPLE.lastRequest')).to.equal(Date.now());
    });

    it('should keep previous data as is (referential transparent)', function () {
      chai.expect(lodash.get(next, 'SAMPLE.data')).to.equal(state.SAMPLE.data);
    });

    it('should set error to null', function () {
      chai.expect(lodash.get(next, 'SAMPLE.error')).to.equal(null);
    });
  });

  describe('FETCH_START handler with `error` property true', function () {
    var next = void 0;
    var time = Date.now();
    var state = {
      SAMPLE: {
        data: {
          key: 'old_value'
        },
        error: {
          message: 'some error has occurred'
        }
      }
    };

    before(function () {
      var api = {
        name: 'SAMPLE',
        endpoint: 'http://example.com',
        requestedAt: time
      };
      var error = { message: 'some new error' };
      next = reducer(state, makeStartErrorAction(api)(error));
    });

    it('should set isFetching to false', function () {
      chai.expect(lodash.get(next, 'SAMPLE.isFetching')).to.equal(false);
    });

    it('should set isInvalidated to true', function () {
      chai.expect(lodash.get(next, 'SAMPLE.isInvalidated')).to.equal(true);
    });

    it('should set lastRequest to requestedAt time', function () {
      chai.expect(lodash.get(next, 'SAMPLE.lastRequest')).to.equal(time);
    });

    it('should keep previous data as is (referential transparent)', function () {
      chai.expect(lodash.get(next, 'SAMPLE.data')).to.equal(state.SAMPLE.data);
    });

    it('should set error to new error', function () {
      chai.expect(lodash.get(next, 'SAMPLE.error')).to.deep.equal({ message: 'some new error' });
    });
  });

  describe('FETCH_COMPLETE handler', function () {
    var next = void 0;
    var state = {
      SAMPLE: {
        data: {
          key: 'old_value'
        },
        error: {
          message: 'some error has occurred'
        }
      }
    };

    before(function () {
      var api = {
        name: 'SAMPLE',
        endpoint: 'http://example.com'
      };
      var json = { key: 'new_value' };
      next = reducer(state, makeSuccessAction(api)(json));
    });

    it('should set isFetching to false', function () {
      chai.expect(lodash.get(next, 'SAMPLE.isFetching')).to.equal(false);
    });

    it('should set isInvalidated to false', function () {
      chai.expect(lodash.get(next, 'SAMPLE.isInvalidated')).to.equal(false);
    });

    it('should set lastResponse = respondedAt', function () {
      chai.expect(lodash.get(next, 'SAMPLE.lastResponse')).to.equal(Date.now());
    });

    it('should set data to new data', function () {
      chai.expect(lodash.get(next, 'SAMPLE.data')).to.deep.equal({ key: 'new_value' });
    });

    it('should set error to null', function () {
      chai.expect(lodash.get(next, 'SAMPLE.error')).to.equal(null);
    });
  });

  describe('FETCH_FAIL handler', function () {
    var next = void 0;
    var state = {
      SAMPLE: {
        data: {
          key: 'old_value'
        },
        lastResponse: 1474877514131,
        error: {
          message: 'some error has occurred'
        }
      }
    };

    before(function () {
      var api = {
        name: 'SAMPLE',
        endpoint: 'http://example.com'
      };
      var json = { error: 'new error' };
      next = reducer(state, makeFailureAction(api)(json));
    });

    it('should set isFetching to false', function () {
      chai.expect(lodash.get(next, 'SAMPLE.isFetching')).to.equal(false);
    });

    it('should set isInvalidated to false', function () {
      chai.expect(lodash.get(next, 'SAMPLE.isInvalidated')).to.equal(true);
    });

    it('should keep old lastResponse', function () {
      chai.expect(lodash.get(next, 'SAMPLE.lastResponse')).to.equal(1474877514131);
    });

    it('should keep old data', function () {
      chai.expect(lodash.get(next, 'SAMPLE.data')).to.equal(state.SAMPLE.data);
    });

    it('should set error to null', function () {
      chai.expect(lodash.get(next, 'SAMPLE.error')).to.deep.equal({ error: 'new error' });
    });
  });
});

describe('utils > get', function () {
  it('should return default value if actual value is undefined', function () {
    chai.expect(get$2(['a', 'b', 'c'], 1)({})).to.equal(1);
  });

  it('should return value in nested object', function () {
    chai.expect(get$2(['a', 'b', 'c'], 1)({ a: { b: { c: 'xxx' } } })).to.equal('xxx');
  });
});
//# sourceMappingURL=tests.js.map
